@@ -108,9 +108,13 @@ this variable will be ignored to honor the mode specified in
 
 (defcustom greader-audiobook-modes '((ereader-mode . "\\W*"))
   "Different treatment of block based on the current major mode.
-Instead of numerical block size, use a string to determine the end of
-each block."
-  :type '(alist :key-type (symbol :tag "mode") :value-type (string)))
+Instead of numerical block size, use a string or function to determine the end
+of each block.
+If you specify a function, that function has to return a cons in which
+car represents the start of the block, and cdr represents the end,
+or nil if there are no more blocks to convert."
+  :type '(alist :key-type (symbol :tag "mode") :value-type (choice
+  (string  function))))
 
 (defcustom greader-audiobook-transcode-wave-files nil
   "If enabled,  transcode original wave files using `ffmpeg'."
@@ -274,7 +278,7 @@ You have certain control of how this happens by configuring
 				   "."
 				   greader-audiobook-transcode-format))))
        (result nil))
-    (setq result (apply 'call-process "ffmpeg" nil "*ffmpeg-output*"
+    (setq result (apply #'call-process "ffmpeg" nil "*ffmpeg-output*"
 			nil ffmpeg-args))
     (unless (eq result 0)
       (error "Error while transcoding, see buffer `*ffmpeg-output*'"))))
@@ -304,7 +308,7 @@ COUNTER represents the current file name."
 								   ".zip"))
 			  (list book-directory)))
 	(result nil))
-    (setq result (apply 'call-process "zip" nil "*audiobook-zip*" nil
+    (setq result (apply #'call-process "zip" nil "*audiobook-zip*" nil
 			zip-args))
     (unless (eq result 0)
       (error "Error while compressing, see buffer *audiobook-zip* for
@@ -417,9 +421,12 @@ buffer without the extension, if any."
     (let ((response (yes-or-no-p "This audiobook already
   exists.  Overwrite it?")))
       (if response
-	  (delete-directory (concat greader-audiobook-base-directory
-				    (file-name-sans-extension (buffer-name)))
-			    t t)
+	  (progn
+	    (unless greader-audiobook-buffer-quietly
+	      (message "removing old audiobook..."))
+	    (delete-directory (concat greader-audiobook-base-directory
+				      (file-name-sans-extension (buffer-name)))
+			      t t))
 	(user-error "Audiobook creation aborted by user"))))
   (unless greader-audiobook-buffer-quietly
     (message "Preparing for conversion (this could take some time...)"))
@@ -500,6 +507,30 @@ buffer without the extension, if any."
 		   (concat greader-audiobook-base-directory
 			   book-directory)))))))
 
+(defvar greader-audiobook-transcode-history nil)
+
+(defun greader-audiobook-retranscode (audiobook-directory new-format)
+  "Transcode an AUDIOBOOK-DIRECTORY into NEW-FORMAT.
+If `greader-audiobook-cancel-intermediate-wave-files is enabled, then
+original files will be deleted."
+
+  (interactive
+   (let ((book-directory (read-directory-name "Audiobook to
+re-transcode (directory): " greader-audiobook-base-directory nil t))
+	 (new-format (read-string "New format: " nil
+				 'greader-audiobook-transcode-history)))
+     (list book-directory new-format)))
+  (let* ((default-directory audiobook-directory)
+	 (greader-audiobook-transcode-format new-format)
+	 (file-list (directory-files default-directory nil
+				    "^[[:digit:]]")))
+    (dolist (file file-list)
+      (unless greader-audiobook-buffer-quietly
+	(message "Re-transcoding file %s..." file))
+      (greader-audiobook-transcode-file file)
+      (when greader-audiobook-cancel-intermediate-wave-files
+	(delete-file file)))
+    (message "re-transcoding finished.")))
 
 (provide 'greader-audiobook)
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
