@@ -162,12 +162,17 @@
 					(filter
 					 . "\%f")
 					(word . ""))
-  "item types and relative prefixes.")
+  "Item types and relative prefixes.")
 
 ;; This function saves the contents of the hash table.
-(defvar greader-dict-directory (concat user-emacs-directory
-				       ".greader-dict/")
+(defvar-local greader-dict-directory (concat user-emacs-directory
+					     ".greader-dict/"
+					     (greader-get-language)
+					     "/")
   "The directory containing greader-dict files.")
+
+(defvar-local greader-dict-local-language (greader-get-language))
+
 (defvar-local greader-dict-filename "greader-dict.global"
   "File name where dictionary definitions are stored.")
 (defvar greader-dict--current-reading-buffer (current-buffer))
@@ -182,6 +187,7 @@
   "C-r d k" #'greader-dict-remove-entry
   "C-r d c" #'greader-dict-change-dictionary
   "C-r d l" #'greader-dict-pronounce-in-other-language
+  "C-r d m" #'greader-dict-modify-key
   "C-r d s" #'greader-dict-save)
 
 (defvar greader-dict--type-file-alternatives '(buffer mode global))
@@ -283,6 +289,8 @@ as a word definition."
     (add-hook 'greader-after-change-language-hook
 	      (lambda ()
 		(when greader-dict-mode
+		  (setq greader-dict-local-language
+			(greader-get-language))
 		  (greader-dict-read-from-dict-file)))))))
 
 ;; THanks to the loved and alwais useful elisp reference.
@@ -334,7 +342,7 @@ as a word definition."
 	       (concat (match-string 1)
 		       (gethash match greader-dictionary)
 		       (match-string 3))))
-	  (replace-match replacement))
+	  (replace-match replacement nil t))
 	(goto-char (point-min))))))
 
 ;; This function adds to the `greader-dictionary' variable the
@@ -349,6 +357,8 @@ If you want to add a partial replacement, you should
 add `\*'to the end of the WORD string parameter."
   ;; We prevent an infinite loop if disallowing that key and values
   ;; are the same.
+  (unless replacement
+    (setq replacement ""))
   (when (string-equal-ignore-case word replacement)
     (user-error "key and value are the same, aborting"))
   (puthash word replacement greader-dictionary)
@@ -408,37 +418,38 @@ Return nil if KEY is not present in `greader-dictionary'."
 		(throw 'key-found result))
 	    nil))))))
 
+(defcustom greader-dict-transform-match-to-word nil
+  "If enabled, every match in the text will be added as a word.
+This option could improve slightly the performance of this framework,
+by adding every match found in the text as a word."
+  :type 'boolean)
+
 (defun greader-dict--get-key-from-word (word)
   "Return key related to WORD, nil otherwise."
+  (unless word
+    (setq word""))
   (setq word (string-trim word))
   (cond
    ((gethash word greader-dictionary)
     word)
    (t
-    (let ((reduced-dictionary (make-hash-table :test 'ignore-case)))
+    (let ((reduced-dictionary (make-hash-table :test 'ignore-case))
+	  (key nil))
       (dolist (item (greader-dict--get-matches 'match))
 	(puthash item (gethash item greader-dictionary)
 		 reduced-dictionary))
-      (let ((key nil))
-	(catch 'matched
-	  (maphash
-	   (lambda (k _v)
-	     (let* ((result (string-remove-suffix
-			     greader-dict-match-indicator k))
-		    (candidate-matches (string-split result "\\W" t)))
-	       (setq candidate-matches (sort candidate-matches
-					     (lambda (s1 s2) (>
-							      (length
-							       s1)
-							      (length
-							       s2)))))
-	       (dolist (candidate candidate-matches)
-		 ;; (message "%s" (concat "matching " candidate " against " word "..."))
-		 (when (string-match candidate word)
-		   ;; (message "Matched!")
-		   (setq key (concat k greader-dict-match-indicator))
-		   (throw 'matched key)))))
-	   reduced-dictionary)))))))
+      (catch 'key-matched
+	(maphash
+	 (lambda (k v)
+	   (setq k
+		 (string-remove-suffix greader-dict-match-indicator k))
+	   (when (string-match k word)
+	     (when greader-dict-transform-match-to-word
+	       (greader-dict--add-match-as-word k word v))
+	     (setq key (concat k greader-dict-match-indicator))
+	     (throw 'key-matched key)))
+	 reduced-dictionary))
+      key      ))))
 
 ;; This function checks that, in the string you pass to it, there are
 ;; effectively words to be replaced. If so, use apis
@@ -499,9 +510,12 @@ Return nil if KEY is not present in `greader-dictionary'."
     (setq greader-dict-filename (buffer-local-value
 				 'greader-dict-filename
 				 greader-dict--current-reading-buffer))
+    (setq greader-dict-local-language (buffer-local-value
+				       'greader-dict-local-language
+				       greader-dict--current-reading-buffer))
     (maphash
      (lambda (k v)
-       (insert (concat "\"" k "\"" "=" v "\n")))
+       (insert "\"" k "\"" "=" v "\n"))
      greader-dictionary)
     (write-region (point-min) (point-max)
 		  (greader-dict--get-file-name)))
@@ -529,7 +543,8 @@ user-error and aborts the reading process."
       (setq greader-filters (buffer-local-value 'greader-filters
 						greader-dict--current-reading-buffer))
       (setq greader-dict-toggle-filters (buffer-local-value
-					 'greader-dict-toggle-filters greader-dict--current-reading-buffer))
+					 'greader-dict-toggle-filters
+					 greader-dict--current-reading-buffer))
       (insert-file-contents (greader-dict--get-file-name))
       (when-let ((lines (string-lines (buffer-string) t)))
 	(dolist (line lines)
@@ -580,7 +595,7 @@ word, it will be added as a match.
 If neither the region is active nor point is on a word, simply asks
 for definition and substitution, without defaults.
 If called with prefix argument, ask for a match.
-In this case you can type a regular expression.
+In this case you can type a partial word or a regular expression.
 You can use regular expressions to, for example, craft filters instead
 of pronunciation rules.
 If the customizable variable
@@ -596,7 +611,11 @@ the current sentence."
       (unless key
 	(user-error "Input is empty: aborting"))
       (setq key (concat key greader-dict-match-indicator))
-      (setq value (read-string (concat "substitute regexp " key "with:
+      (setq value (read-string (concat "substitute match "
+				       (string-remove-suffix
+					greader-dict-match-indicator
+					key)
+				       " with:
 ")
 			       nil nil(gethash key greader-dictionary)))
       (greader-dict-add key value))
@@ -620,10 +639,13 @@ the current sentence."
 modify: "
 				   nil
 				   nil
-				   (append (list default-word)(when
-								  greader-dict-include-sentences-in-defaults
-								(greader-dict--get-word-alternatives
-								 (greader-get-sentence)))
+				   (append (list
+					    (substring-no-properties
+					     default-word))
+					   (when
+					       greader-dict-include-sentences-in-defaults
+					     (greader-dict--get-word-alternatives
+					      (greader-get-sentence)))
 					   (greader-dict--get-matches
 					    'word))))
 	    (setq value (read-string (concat "substitute word " key
@@ -635,7 +657,9 @@ modify: "
 	(setq value (read-string (concat "substitute " key " with: ")
 				 nil nil
 				 (gethash key greader-dictionary)))
-	(greader-dict-add key value))))))
+	(greader-dict-add key value)))))
+  (deactivate-mark t))
+
 (declare-function hash-table-keys nil)
 (defun greader-dict-remove-entry (key)
   "Remove KEY from the dictionary.
@@ -668,9 +692,23 @@ argument, only if `greader-dict-mode' is enabled."
 (declare-function greader-get-language nil)
 (defun greader-dict--get-file-name ()
   "Return the absolute path of current dictionary file."
-  (concat greader-dict-directory (greader-get-language) "/"
-	  (buffer-local-value 'greader-dict-filename
-			      greader-dict--current-reading-buffer)))
+  (let ((language-part (car (last (split-string greader-dict-directory
+						"/" t)))))
+    (unless (equal language-part (buffer-local-value
+				  'greader-dict-local-language
+				  greader-dict--current-reading-buffer))
+      (setq greader-dict-directory (string-remove-suffix (concat
+							  language-part
+							  "/")
+							 greader-dict-directory))
+      (setq greader-dict-directory
+	    (concat greader-dict-directory (buffer-local-value
+					    'greader-dict-local-language
+					    greader-dict--current-reading-buffer)
+		    "/"))))
+  (concat greader-dict-directory (buffer-local-value
+				  'greader-dict-filename
+				  greader-dict--current-reading-buffer)))
 
 (defun greader-dict--set-file (type)
   "Set `greader-dict-filename' according to TYPE.
@@ -769,8 +807,7 @@ asked."
       (greader-dict--set-file (intern new-dict))
       (unless (file-exists-p (greader-dict--get-file-name))
 	(shell-command-to-string
-	 (concat "touch " greader-dict-filename)))
-      (greader-dict--update)
+	 (concat "touch " (greader-dict--get-file-name))))
       (greader-dict--update))))
 
 ;; (remove-hook 'buffer-list-update-hook #'greader-dict--update)))))
@@ -847,18 +884,20 @@ If TYPE is `all', all items in the current dictionary will be included."
      (lambda (s1 s2)
        (string-greaterp s2 s1)))))
 (defun greader-dict-modify-key (arg)
-  "Modify a key (_NOT_ a value associated with it!).
+  "Modify a key \(_NOT_ a value associated with it!\).
 While `greader-dict-add-entry can modify either keys or associated
 values, `greader-dict-modify-key' allows you to modify the key
 itself, without modifying the associated value.
 if prefix ARG is non-nil, then this command proposes only keys that
-are classified as matches."
+are classified as matches.
+When called without a prefix, it modifies only keys that are
+classified as words."
   (interactive "P")
   (let ((key (read-string "key to modify: " nil nil (if arg
 							(greader-dict--get-matches
 							 'match)
 						      (greader-dict--get-matches
-						       'all))))
+						       'word))))
 	(new-key nil))
     (unless key
       (user-error "Key not valid"))
@@ -878,7 +917,9 @@ are classified as matches."
   (if-let ((alternatives text))
       (progn
 	(setq alternatives nil)
-	(dolist (word (split-string text "\\W" t))
+	(dolist
+	    (word
+	     (split-string (substring-no-properties text) "\\W" t))
 	  (unless (member word alternatives)
 	    (push word alternatives)))
 	(reverse alternatives))
@@ -925,6 +966,8 @@ in the current sentence."
 		(gethash (concat key greader-dict-filter-indicator)
 			 greader-dictionary))))
      (list key value)))
+  (while (string-match "\n" key)
+    (setq key (replace-match "[[:space:]]" nil nil key)))
   (greader-dict-add (concat key greader-dict-filter-indicator) value)
   (greader-dict--filter-init))
 
@@ -981,14 +1024,40 @@ hash table."
   "Apply filters defined in sequence to the current buffer."
   (maphash
    (lambda (k v)
+     (setq k (string-remove-suffix greader-dict-filter-indicator k))
+     (setq k (string-trim-right k))
      (save-excursion
        (goto-char (point-min))
-       (while (re-search-forward (string-remove-suffix
-				  greader-dict-filter-indicator k)
-				 nil t)
+       (while (re-search-forward k nil t)
 	 (replace-match v))))
    greader-filters))
 
+(defun greader-dict--add-match-as-word (key word replacement &optional
+					    original-word)
+  "Add WORD and REPLACEMENT to the current dictionary.
+This function is used internally, please use the normal entry-points
+to add your own items to the dictionary.
+This function works only when REPLACEMENT contains only word
+constituents."
+
+  (unless (string-match "\\W" replacement nil t)
+    (let (value start end)
+      (string-match key word)
+      (setq start (match-beginning 0))
+      (setq end (match-end 0))
+      (when (> start 0)
+	(setq value (concat (substring word 0 start))))
+      (setq value (concat value replacement))
+      (when (> (length word) (length value))
+	(setq value (concat value (substring word end))))
+      (if (string-match key value)
+	  (progn
+	    (setq original-word word)
+	    (greader-dict--add-match-as-word key value replacement
+					     original-word))
+	(setq key (or original-word word))
+	(greader-dict-add key value)))))
+
 (provide 'greader-dict)
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; greader-dict.el ends here
