@@ -7,7 +7,7 @@
 ;; Keywords: tools, accessibility
 ;; URL: https://gitlab.com/michelangelo-rodriguez/greader
 
-;; Version: 0.11.10
+;; Version: 0.11.18
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -236,14 +236,14 @@ if set to t, when you call function `greader-read', that function sets a
 
 (defun greader-set-reading-keymap ()
   "Set greader's keymap when reading."
-  (greader-mode -1)
-  (greader-reading-mode t))
+  (setq greader-mode nil)
+  (setq greader-reading-mode t))
 
 (defun greader-set-greader-keymap ()
   "Set greader's keymap when not reading."
 
-  (greader-mode t)
-  (greader-reading-mode -1))
+  (setq greader-mode t)
+  (setq greader-reading-mode nil))
 
 (define-obsolete-variable-alias 'greader-map 'greader-mode-map "2022")
 (defvar greader-mode-map
@@ -384,9 +384,14 @@ This only happens if the variables `greader-start-region' and
   (if arg
       (funcall greader-current-backend command arg)
     (funcall greader-current-backend command)))
-(defvar
-  greader-backend-filename
+(defun greader-load-backends ()
+  "Load backends taken from `greader-backends'."
+  (mapcar #'require greader-backends))
+(greader-load-backends)
+
+(defvar   greader-backend-filename
   (greader-call-backend 'executable))
+
 (defvar greader-backend `(,greader-backend-filename))
 (defvar greader-orig-buffer nil)
 (defvar greader-dissoc-buffer "*Dissociation*")
@@ -426,9 +431,7 @@ available backends."
   (message "Current back-end is %s"
            (get greader-current-backend 'greader-backend-name)))
 
-(defun greader-load-backends ()
-  "Load backends taken from `greader-backends'."
-  (mapcar #'require greader-backends))
+
 
 (defun greader-read-asynchronous (txt)
   "Read the text given in TXT."
@@ -1224,10 +1227,10 @@ When called from a function, you should specify SRC and DST, even if
   (unless src
     (setq src (thing-at-point 'word t))
     (unless src
-      (setq src (read-string "Word to add:"))))
+      (setq src (read-string "Word to add: "))))
 
   (when (listp src)
-    (setq src (read-string "word to add:"))
+    (setq src (read-string "word to add: "))
     (if (equal src "")
 	(setq src (thing-at-point 'word t))))
   (unless dst
@@ -1242,7 +1245,7 @@ When called from a function, you should specify SRC and DST, even if
 		   greader-compile-default-source))))
     (with-current-buffer (find-file-noselect lang-file)
       (goto-char (point-max))
-      (insert (concat src " " dst "\n"))
+      (insert src " " dst "\n")
       (save-buffer)
       (unless greader-compile-mode
 	(greader-compile)))))
@@ -1724,7 +1727,7 @@ When called interactively, use the current position in the buffer."
   "restart reading of buffer or region."
   (if greader-region-mode
       (goto-char (region-beginning))
-    (goto-char (point-min)))
+    (goto-char greader-study-start-position))
   (greader-read) t)
 
 ;; greader-study-mode
